@@ -24,14 +24,6 @@ static struct option opts[] = {
 	{ .name = 0 }
 };
 
-
-static void
-init(struct ipt_entry_match *m, unsigned int *nfcache)
-{
-	*nfcache |= NFC_UNKNOWN;
-}
-
-
 static int
 parse(int c, char **argv, int invert, unsigned int *flags,
       const struct ipt_entry *entry, unsigned int *nfcache,
@@ -94,17 +86,16 @@ save(const struct ipt_ip *ip,
 
 
 static struct iptables_match condition = {
-	.name = "condition",
-	.version = IPTABLES_VERSION,
-	.size = IPT_ALIGN(sizeof(struct condition_info)),
-	.userspacesize = IPT_ALIGN(sizeof(struct condition_info)),
-	.help = &help,
-	.init = &init,
-	.parse = &parse,
-	.final_check = &final_check,
-	.print = &print,
-	.save = &save,
-	.extra_opts = opts
+	.name 		= "condition",
+	.version 	= IPTABLES_VERSION,
+	.size 		= IPT_ALIGN(sizeof(struct condition_info)),
+	.userspacesize 	= IPT_ALIGN(sizeof(struct condition_info)),
+	.help 		= &help,
+	.parse 		= &parse,
+	.final_check	= &final_check,
+	.print 		= &print,
+	.save 		= &save,
+	.extra_opts 	= opts
 };
 
 
